@@ -1,35 +1,29 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
-
 /*
- *  (LIBGLABELS) Template library for GLABELS
+ *  xml-template.c
+ *  Copyright (C) 2001-2009  Jim Evins <evins@snaught.com>.
  *
- *  xml-template.c:  template xml module
+ *  This file is part of libglabels.
  *
- *  Copyright (C) 2001-2006  Jim Evins <evins@snaught.com>.
+ *  libglabels is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
  *
- *  This file is part of the LIBGLABELS library.
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
- *
- *  This library is distributed in the hope that it will be useful,
+ *  libglabels is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU Library General Public
- *  License along with this library; if not, write to the Free
- *  Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
- *  MA 02111-1307, USA
+ *  You should have received a copy of the GNU Lesser General Public License
+ *  along with libglabels.  If not, see <http://www.gnu.org/licenses/>.
  */
+
 #include <config.h>
 
 #include "xml-template.h"
 
 #include <glib/gi18n.h>
-#include <glib/gmessages.h>
+#include <glib.h>
 #include <string.h>
 #include <libintl.h>
 
@@ -53,6 +47,8 @@ static void  xml_parse_meta_node            (xmlNodePtr              label_node,
 					     lglTemplate            *template);
 static void  xml_parse_label_rectangle_node (xmlNodePtr              label_node,
 					     lglTemplate            *template);
+static void  xml_parse_label_ellipse_node   (xmlNodePtr              label_node,
+					     lglTemplate            *template);
 static void  xml_parse_label_round_node     (xmlNodePtr              label_node,
 					     lglTemplate            *template);
 static void  xml_parse_label_cd_node        (xmlNodePtr              label_node,
@@ -67,10 +63,13 @@ static void  xml_parse_markup_circle_node   (xmlNodePtr              markup_node
 					     lglTemplateFrame       *frame);
 static void  xml_parse_markup_rect_node     (xmlNodePtr              markup_node,
 					     lglTemplateFrame       *frame);
+static void  xml_parse_markup_ellipse_node  (xmlNodePtr              markup_node,
+					     lglTemplateFrame       *frame);
 static void  xml_parse_alias_node           (xmlNodePtr              alias_node,
 					     lglTemplate            *template);
 
-static void  xml_create_meta_node           (const gchar                  *category,
+static void  xml_create_meta_node           (const gchar                  *attr,
+                                             const gchar                  *value,
 					     xmlNodePtr                    root,
 					     const xmlNsPtr                ns);
 static void  xml_create_label_node          (const lglTemplateFrame       *frame,
@@ -91,6 +90,9 @@ static void  xml_create_markup_circle_node  (const lglTemplateMarkup      *circl
 static void  xml_create_markup_rect_node    (const lglTemplateMarkup      *circle,
 					     xmlNodePtr                    root,
 					     const xmlNsPtr                ns);
+static void  xml_create_markup_ellipse_node (const lglTemplateMarkup      *circle,
+					     xmlNodePtr                    root,
+					     const xmlNsPtr                ns);
 static void  xml_create_alias_node          (const lglTemplateAlias       *alias,
 					     xmlNodePtr                    root,
 					     const xmlNsPtr                ns);
@@ -102,37 +104,32 @@ static void  xml_create_alias_node          (const lglTemplateAlias       *alias
  *
  * Read glabels templates from template file.
  *
- * Returns: a list of #lglTemplate structures.
- *
  */
-GList *
+void
 lgl_xml_template_read_templates_from_file (const gchar *utf8_filename)
 {
 	gchar      *filename;
 	xmlDocPtr   templates_doc;
-	GList      *templates = NULL;
 
 	LIBXML_TEST_VERSION;
 
 	filename = g_filename_from_utf8 (utf8_filename, -1, NULL, NULL, NULL);
 	if (!filename) {
 		g_message ("Utf8 filename conversion error");
-		return NULL;
+		return;
 	}
 
 	templates_doc = xmlParseFile (filename);
 	if (!templates_doc) {
 		g_message ("\"%s\" is not a glabels template file (not XML)",
 		      filename);
-		return templates;
+		return;
 	}
 
-	templates = lgl_xml_template_parse_templates_doc (templates_doc);
+	lgl_xml_template_parse_templates_doc (templates_doc);
 
 	g_free (filename);
 	xmlFreeDoc (templates_doc);
-
-	return templates;
 }
 
 
@@ -142,14 +139,11 @@ lgl_xml_template_read_templates_from_file (const gchar *utf8_filename)
  *
  * Read glabels templates from a libxml #xmlDocPtr tree.
  *
- * Returns: a list of #lglTemplate structures.
- *
  */
-GList *
+void
 lgl_xml_template_parse_templates_doc (const xmlDocPtr templates_doc)
 {
 	
-	GList       *templates = NULL;
 	xmlNodePtr   root, node;
 	lglTemplate *template;
 
@@ -159,19 +153,20 @@ lgl_xml_template_parse_templates_doc (const xmlDocPtr templates_doc)
 	if (!root || !root->name) {
 		g_message ("\"%s\" is not a glabels template file (no root node)",
 			   templates_doc->URL);
-		return templates;
+		return;
 	}
 	if (!lgl_xml_is_node (root, "Glabels-templates")) {
 		g_message ("\"%s\" is not a glabels template file (wrong root node)",
 		      templates_doc->URL);
-		return templates;
+		return;
 	}
 
 	for (node = root->xmlChildrenNode; node != NULL; node = node->next) {
 
 		if (lgl_xml_is_node (node, "Template")) {
 			template = lgl_xml_template_parse_template_node (node);
-			templates = g_list_append (templates, template);
+                        _lgl_db_register_template_internal (template);
+                        lgl_template_free (template);
 		} else {
 			if ( !xmlNodeIsText(node) ) {
 				if (!lgl_xml_is_node (node,"comment")) {
@@ -180,8 +175,6 @@ lgl_xml_template_parse_templates_doc (const xmlDocPtr templates_doc)
 			}
 		}
 	}
-
-	return templates;
 }
 
 
@@ -200,13 +193,16 @@ lgl_xml_template_parse_template_node (const xmlNodePtr template_node)
 	gchar                 *brand;
         gchar                 *part;
 	gchar                 *name;
+        gchar                 *equiv_part;
 	gchar                 *description;
 	gchar                 *paper_id;
 	gdouble                page_width, page_height;
-	lglPaper               *paper = NULL;
+	lglPaper              *paper = NULL;
 	lglTemplate           *template;
 	xmlNodePtr             node;
         gchar                **v;
+        lglTemplateFrame      *frame;
+
 
 	brand = lgl_xml_get_prop_string (template_node, "brand", NULL);
 	part  = lgl_xml_get_prop_string (template_node, "part", NULL);
@@ -228,6 +224,10 @@ lgl_xml_template_parse_template_node (const xmlNodePtr template_node)
                 }
         }
 
+
+        equiv_part = lgl_xml_get_prop_string (template_node, "equiv", NULL);
+
+
 	description = lgl_xml_get_prop_i18n_string (template_node, "description", NULL);
 	paper_id = lgl_xml_get_prop_string (template_node, "size", NULL);
 
@@ -260,8 +260,23 @@ lgl_xml_template_parse_template_node (const xmlNodePtr template_node)
 		paper = NULL;
 	}
 
-	template = lgl_template_new (brand, part, description,
-                                     paper_id, page_width, page_height);
+
+        if (!equiv_part)
+        {
+                template = lgl_template_new (brand, part, description,
+                                             paper_id, page_width, page_height);
+        }
+        else
+        {
+                template = lgl_template_new_from_equiv (brand, part, equiv_part);
+
+                if (!template)
+                {
+			g_message (_("Forward references not supported."));
+                        return NULL;
+                }
+        }
+
 
 	for (node = template_node->xmlChildrenNode; node != NULL;
 	     node = node->next) {
@@ -269,6 +284,8 @@ lgl_xml_template_parse_template_node (const xmlNodePtr template_node)
 			xml_parse_meta_node (node, template);
 		} else if (lgl_xml_is_node (node, "Label-rectangle")) {
 			xml_parse_label_rectangle_node (node, template);
+		} else if (lgl_xml_is_node (node, "Label-ellipse")) {
+			xml_parse_label_ellipse_node (node, template);
 		} else if (lgl_xml_is_node (node, "Label-round")) {
 			xml_parse_label_round_node (node, template);
 		} else if (lgl_xml_is_node (node, "Label-cd")) {
@@ -286,9 +303,31 @@ lgl_xml_template_parse_template_node (const xmlNodePtr template_node)
 
 	g_free (brand);
 	g_free (part);
+	g_free (equiv_part);
 	g_free (description);
 	g_free (paper_id);
 
+        /*
+         * Create a default full-page frame, if a known frame type was not found.
+         */
+        if ( template->frames == NULL )
+        {
+                g_message ("%s %s: missing valid frame node", template->brand, template->part);
+                frame = lgl_template_frame_rect_new ("0", page_width, page_height, 0, 0, 0);
+                lgl_template_frame_add_layout (frame, lgl_template_layout_new (1, 1, 0, 0, 0, 0));
+                lgl_template_add_frame (template, frame);
+        }
+
+        /*
+         * Create a default 1x1 layout, if layout is missing.
+         */
+        frame = (lglTemplateFrame *)template->frames->data;
+        if ( frame->all.layouts == NULL )
+        {
+                g_message ("%s %s: missing layout node", template->brand, template->part);
+                lgl_template_frame_add_layout (frame, lgl_template_layout_new (1, 1, 0, 0, 0, 0));
+        }
+
 	return template;
 }
 
@@ -299,17 +338,25 @@ static void
 xml_parse_meta_node (xmlNodePtr   meta_node,
 		     lglTemplate *template)
 {
+        gchar               *product_url;
 	gchar               *category;
 
-	category = lgl_xml_get_prop_string (meta_node, "category", NULL);
+        product_url = lgl_xml_get_prop_string (meta_node, "product_url", NULL);
+        if ( product_url != NULL )
+        {
+                g_free (template->product_url);
+                template->product_url = product_url;
+        }
 
-	if (category != NULL)
+	category = lgl_xml_get_prop_string (meta_node, "category", NULL);
+	if ( category != NULL )
 	{
 		lgl_template_add_category (template, category);
 		g_free (category);
 	}
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML Template->Label-rectangle Node.                      */
 /*--------------------------------------------------------------------------*/
@@ -354,6 +401,8 @@ xml_parse_label_rectangle_node (xmlNodePtr   label_node,
 			xml_parse_markup_circle_node (node, frame);
 		} else if (lgl_xml_is_node (node, "Markup-rect")) {
 			xml_parse_markup_rect_node (node, frame);
+		} else if (lgl_xml_is_node (node, "Markup-ellipse")) {
+			xml_parse_markup_ellipse_node (node, frame);
 		} else if (!xmlNodeIsText (node)) {
 			if (!lgl_xml_is_node (node, "comment")) {
 				g_message ("bad node =  \"%s\"",node->name);
@@ -364,6 +413,54 @@ xml_parse_label_rectangle_node (xmlNodePtr   label_node,
 	g_free (id);
 }
 
+
+/*--------------------------------------------------------------------------*/
+/* PRIVATE.  Parse XML Template->Label-ellipse Node.                        */
+/*--------------------------------------------------------------------------*/
+static void
+xml_parse_label_ellipse_node (xmlNodePtr   label_node,
+                              lglTemplate *template)
+{
+	gchar               *id;
+	gdouble              waste;
+	gdouble              w, h;
+	lglTemplateFrame    *frame;
+	xmlNodePtr           node;
+
+	id      = lgl_xml_get_prop_string (label_node, "id", NULL);
+
+	w       = lgl_xml_get_prop_length (label_node, "width", 0);
+	h       = lgl_xml_get_prop_length (label_node, "height", 0);
+        waste   = lgl_xml_get_prop_length (label_node, "waste", 0);
+
+	frame = lgl_template_frame_ellipse_new ((gchar *)id, w, h, waste);
+	lgl_template_add_frame (template, frame);
+
+	for (node = label_node->xmlChildrenNode; node != NULL;
+	     node = node->next) {
+		if (lgl_xml_is_node (node, "Layout")) {
+			xml_parse_layout_node (node, frame);
+		} else if (lgl_xml_is_node (node, "Markup-margin")) {
+			xml_parse_markup_margin_node (node, frame);
+		} else if (lgl_xml_is_node (node, "Markup-line")) {
+			xml_parse_markup_line_node (node, frame);
+		} else if (lgl_xml_is_node (node, "Markup-circle")) {
+			xml_parse_markup_circle_node (node, frame);
+		} else if (lgl_xml_is_node (node, "Markup-rect")) {
+			xml_parse_markup_rect_node (node, frame);
+		} else if (lgl_xml_is_node (node, "Markup-ellipse")) {
+			xml_parse_markup_ellipse_node (node, frame);
+		} else if (!xmlNodeIsText (node)) {
+			if (!lgl_xml_is_node (node, "comment")) {
+				g_message ("bad node =  \"%s\"",node->name);
+			}
+		}
+	}
+
+	g_free (id);
+}
+
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML Template->Label-round Node.                          */
 /*--------------------------------------------------------------------------*/
@@ -396,6 +493,8 @@ xml_parse_label_round_node (xmlNodePtr   label_node,
 			xml_parse_markup_circle_node (node, frame);
 		} else if (lgl_xml_is_node (node, "Markup-rect")) {
 			xml_parse_markup_rect_node (node, frame);
+		} else if (lgl_xml_is_node (node, "Markup-ellipse")) {
+			xml_parse_markup_ellipse_node (node, frame);
 		} else if (!xmlNodeIsText (node)) {
 			if (!lgl_xml_is_node (node, "comment")) {
 				g_message ("bad node =  \"%s\"",node->name);
@@ -441,6 +540,8 @@ xml_parse_label_cd_node (xmlNodePtr   label_node,
 			xml_parse_markup_circle_node (node, frame);
 		} else if (lgl_xml_is_node (node, "Markup-rect")) {
 			xml_parse_markup_rect_node (node, frame);
+		} else if (lgl_xml_is_node (node, "Markup-ellipse")) {
+			xml_parse_markup_ellipse_node (node, frame);
 		} else if (!xmlNodeIsText (node)) {
 			if (!lgl_xml_is_node (node, "comment")) {
 				g_message ("bad node =  \"%s\"",node->name);
@@ -564,6 +665,7 @@ xml_parse_markup_circle_node (xmlNodePtr          markup_node,
 
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML Template->Label->Markup-rect Node.                   */
 /*--------------------------------------------------------------------------*/
@@ -593,6 +695,36 @@ xml_parse_markup_rect_node (xmlNodePtr          markup_node,
 
 }
 
+
+/*--------------------------------------------------------------------------*/
+/* PRIVATE.  Parse XML Template->Label->Markup-ellipse Node.                */
+/*--------------------------------------------------------------------------*/
+static void
+xml_parse_markup_ellipse_node (xmlNodePtr          markup_node,
+                               lglTemplateFrame   *frame)
+{
+	gdouble     x1, y1, w, h;
+	xmlNodePtr  node;
+
+	x1 = lgl_xml_get_prop_length (markup_node, "x1", 0);
+	y1 = lgl_xml_get_prop_length (markup_node, "y1", 0);
+	w  = lgl_xml_get_prop_length (markup_node, "w", 0);
+	h  = lgl_xml_get_prop_length (markup_node, "h", 0);
+
+	lgl_template_frame_add_markup (frame, lgl_template_markup_ellipse_new (x1, y1, w, h));
+
+	for (node = markup_node->xmlChildrenNode; node != NULL;
+	     node = node->next) {
+		if (!xmlNodeIsText (node)) {
+			if (!lgl_xml_is_node (node, "comment")) {
+				g_message ("bad node =  \"%s\"",node->name);
+			}
+		}
+	}
+
+}
+
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML Template->Alias Node.                                */
 /*--------------------------------------------------------------------------*/
@@ -752,9 +884,10 @@ lgl_xml_template_create_template_node (const lglTemplate *template,
 			xml_create_alias_node ( alias, node, ns );
 		}
 	}
+        xml_create_meta_node ("product_url", template->product_url, node, ns );
 	for ( p=template->category_ids; p != NULL; p=p->next )
         {
-                xml_create_meta_node ( p->data, node, ns );
+                xml_create_meta_node ( "category", p->data, node, ns );
 	}
 	for ( p=template->frames; p != NULL; p=p->next )
         {
@@ -765,17 +898,21 @@ lgl_xml_template_create_template_node (const lglTemplate *template,
 }
 
 /*--------------------------------------------------------------------------*/
-/* PRIVATE.  Add XML Template->Meta Node.                                   */
+/* PRIVATE.  Add XML Template->Meta Node with category.                     */
 /*--------------------------------------------------------------------------*/
 static void
-xml_create_meta_node (const gchar      *category,
-		      xmlNodePtr        root,
-		      const xmlNsPtr    ns)
+xml_create_meta_node (const gchar      *attr,
+                      const gchar      *value,
+                      xmlNodePtr        root,
+                      const xmlNsPtr    ns)
 {
 	xmlNodePtr node;
 
-	node = xmlNewChild (root, ns, (xmlChar *)"Meta", NULL);
-	lgl_xml_set_prop_string (node, "category", category);
+        if ( value != NULL )
+        {
+                node = xmlNewChild (root, ns, (xmlChar *)"Meta", NULL);
+                lgl_xml_set_prop_string (node, attr, value);
+        }
 
 }
 
@@ -804,6 +941,14 @@ xml_create_label_node (const lglTemplateFrame  *frame,
 		lgl_xml_set_prop_length (node, "y_waste", frame->rect.y_waste);
 		break;
 
+	case LGL_TEMPLATE_FRAME_SHAPE_ELLIPSE:
+		node = xmlNewChild(root, ns, (xmlChar *)"Label-ellipse", NULL);
+		lgl_xml_set_prop_string (node, "id",     frame->all.id);
+		lgl_xml_set_prop_length (node, "width",  frame->ellipse.w);
+		lgl_xml_set_prop_length (node, "height", frame->ellipse.h);
+		lgl_xml_set_prop_length (node, "waste",  frame->ellipse.waste);
+		break;
+
 	case LGL_TEMPLATE_FRAME_SHAPE_ROUND:
 		node = xmlNewChild(root, ns, (xmlChar *)"Label-round", NULL);
 		lgl_xml_set_prop_string (node, "id",      frame->all.id);
@@ -847,6 +992,9 @@ xml_create_label_node (const lglTemplateFrame  *frame,
 		case LGL_TEMPLATE_MARKUP_RECT:
 			xml_create_markup_rect_node (markup, node, ns);
 			break;
+		case LGL_TEMPLATE_MARKUP_ELLIPSE:
+			xml_create_markup_ellipse_node (markup, node, ns);
+			break;
 		default:
 			g_message ("Unknown markup type");
 			break;
@@ -933,6 +1081,7 @@ xml_create_markup_circle_node (const lglTemplateMarkup *markup,
 
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Template->Label->Markup-rect Node.                     */
 /*--------------------------------------------------------------------------*/
@@ -953,6 +1102,27 @@ xml_create_markup_rect_node (const lglTemplateMarkup *markup,
 
 }
 
+
+/*--------------------------------------------------------------------------*/
+/* PRIVATE.  Add XML Template->Label->Markup-ellipse Node.                  */
+/*--------------------------------------------------------------------------*/
+static void
+xml_create_markup_ellipse_node (const lglTemplateMarkup *markup,
+                                xmlNodePtr               root,
+                                const xmlNsPtr           ns)
+{
+	xmlNodePtr  node;
+
+	node = xmlNewChild(root, ns, (xmlChar *)"Markup-ellipse", NULL);
+
+	lgl_xml_set_prop_length (node, "x1", markup->ellipse.x1);
+	lgl_xml_set_prop_length (node, "y1", markup->ellipse.y1);
+	lgl_xml_set_prop_length (node, "w",  markup->ellipse.w);
+	lgl_xml_set_prop_length (node, "h",  markup->ellipse.h);
+
+}
+
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Template->Alias Node.                                  */
 /*--------------------------------------------------------------------------*/
@@ -970,3 +1140,13 @@ xml_create_alias_node (const lglTemplateAlias *alias,
 
 }
 
+
+
+/*
+ * Local Variables:       -- emacs
+ * mode: C                -- emacs
+ * c-basic-offset: 8      -- emacs
+ * tab-width: 8           -- emacs
+ * indent-tabs-mode: nil  -- emacs
+ * End:                   -- emacs
+ */
