@@ -1,64 +1,65 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
-
 /*
- *  (GLABELS) Label and Business Card Creation program for GNOME
+ *  object-editor-bc-page.c
+ *  Copyright (C) 2003-2009  Jim Evins <evins@snaught.com>.
  *
- *  object-editor.c:  object properties editor module
+ *  This file is part of gLabels.
  *
- *  Copyright (C) 2003  Jim Evins <evins@snaught.com>.
- *
- *  This program is free software; you can redistribute it and/or modify
+ *  gLabels is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  gLabels is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ *  along with gLabels.  If not, see <http://www.gnu.org/licenses/>.
  */
+
 #include <config.h>
 
 #include "object-editor.h"
 
 #include <glib/gi18n.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtktogglebutton.h>
+#include <gtk/gtk.h>
 #include <math.h>
 
 #include "prefs.h"
-#include "mygal/widget-color-combo.h"
+#include "color-combo.h"
 #include "color.h"
-#include "util.h"
+#include "field-button.h"
+#include "combo-util.h"
+#include "builder-util.h"
 
 #include "object-editor-private.h"
 
 #include "debug.h"
 
+
 /*===========================================*/
 /* Private macros                            */
 /*===========================================*/
 
+
 /*===========================================*/
 /* Private data types                        */
 /*===========================================*/
 
+
 /*===========================================*/
 /* Private globals                           */
 /*===========================================*/
 
+
 /*===========================================*/
 /* Local function prototypes                 */
 /*===========================================*/
 
-static void style_changed_cb     (glObjectEditor       *editor);
-static void bc_radio_toggled_cb  (glObjectEditor       *editor);
+static void style_changed_cb            (glObjectEditor       *editor);
+static void bc_radio_toggled_cb         (glObjectEditor       *editor);
+static void data_digits_spin_changed_cb (glObjectEditor       *editor);
 
 
 /*--------------------------------------------------------------------------*/
@@ -72,40 +73,41 @@ gl_object_editor_prepare_bc_page (glObjectEditor       *editor)
 	gl_debug (DEBUG_EDITOR, "START");
 
 	/* Extract widgets from XML tree. */
-	editor->priv->bc_page_vbox =
-		glade_xml_get_widget (editor->priv->gui, "bc_page_vbox");
-	editor->priv->bc_style_combo =
-		glade_xml_get_widget (editor->priv->gui, "bc_style_combo");
-	editor->priv->bc_text_check =
-		glade_xml_get_widget (editor->priv->gui, "bc_text_check");
-	editor->priv->bc_cs_check =
-		glade_xml_get_widget (editor->priv->gui, "bc_cs_check");
-	editor->priv->bc_color_combo =
-		glade_xml_get_widget (editor->priv->gui, "bc_color_combo");
-	editor->priv->bc_key_combo = 
-		glade_xml_get_widget (editor->priv->gui, "bc_key_combo");	
-	editor->priv->bc_key_radio = 
-		glade_xml_get_widget (editor->priv->gui, "bc_key_radio");	
-	editor->priv->bc_color_radio = 
-		glade_xml_get_widget (editor->priv->gui, "bc_color_radio");	
-	editor->priv->data_format_label =
-		glade_xml_get_widget (editor->priv->gui, "data_format_label");
-	editor->priv->data_ex_label =
-		glade_xml_get_widget (editor->priv->gui, "data_ex_label");
-	editor->priv->data_digits_label =
-		glade_xml_get_widget (editor->priv->gui, "data_digits_label");
-	editor->priv->data_digits_spin =
-		glade_xml_get_widget (editor->priv->gui, "data_digits_spin");
+        gl_builder_util_get_widgets (editor->priv->builder,
+                                     "bc_page_vbox",      &editor->priv->bc_page_vbox,
+                                     "bc_style_combo",    &editor->priv->bc_style_combo,
+                                     "bc_text_check",     &editor->priv->bc_text_check,
+                                     "bc_cs_check",       &editor->priv->bc_cs_check,
+                                     "bc_color_hbox",     &editor->priv->bc_color_hbox,
+                                     "bc_key_hbox",       &editor->priv->bc_key_hbox,
+                                     "bc_key_radio",      &editor->priv->bc_key_radio,
+                                     "bc_color_radio",    &editor->priv->bc_color_radio,
+                                     "data_format_label", &editor->priv->data_format_label,
+                                     "data_ex_label",     &editor->priv->data_ex_label,
+                                     "data_digits_label", &editor->priv->data_digits_label,
+                                     "data_digits_spin",  &editor->priv->data_digits_spin,
+                                     NULL);
 
 	editor->priv->data_format_fixed_flag = FALSE;
 
-	gl_util_combo_box_add_text_model ( GTK_COMBO_BOX(editor->priv->bc_style_combo));
-	gl_util_combo_box_add_text_model ( GTK_COMBO_BOX(editor->priv->bc_key_combo));
+	editor->priv->bc_color_combo = gl_color_combo_new (_("Default"),
+                                                           GL_COLOR_BC_DEFAULT,
+                                                           gl_prefs_model_get_default_line_color (gl_prefs));
+        gtk_box_pack_start (GTK_BOX (editor->priv->bc_color_hbox),
+                            editor->priv->bc_color_combo,
+                            FALSE, FALSE, 0);
+
+        editor->priv->bc_key_combo = gl_field_button_new (NULL);
+        gtk_box_pack_start (GTK_BOX (editor->priv->bc_key_hbox),
+                            editor->priv->bc_key_combo,
+                            TRUE, TRUE, 0);
+
+	gl_combo_util_add_text_model ( GTK_COMBO_BOX(editor->priv->bc_style_combo));
 
 	/* Load barcode styles */
 	styles = gl_barcode_get_styles_list ();
-	gl_util_combo_box_set_strings (GTK_COMBO_BOX(editor->priv->bc_style_combo),
-				       styles);
+	gl_combo_util_set_strings (GTK_COMBO_BOX(editor->priv->bc_style_combo),
+                                   styles);
 	gl_barcode_free_styles_list (styles);
 
 	/* Modify widgets */
@@ -147,12 +149,13 @@ gl_object_editor_prepare_bc_page (glObjectEditor       *editor)
 				  G_OBJECT (editor));
 	g_signal_connect_swapped (G_OBJECT (editor->priv->data_digits_spin),
 				  "changed",
-				  G_CALLBACK (gl_object_editor_changed_cb),
+				  G_CALLBACK (data_digits_spin_changed_cb),
 				  G_OBJECT (editor));
 
 	gl_debug (DEBUG_EDITOR, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Callback for when style has changed.                           */
 /*--------------------------------------------------------------------------*/
@@ -184,12 +187,8 @@ style_changed_cb (glObjectEditor       *editor)
                                           gl_barcode_csum_optional (id));
                                                                                 
 		editor->priv->data_format_fixed_flag = !gl_barcode_can_freeform (id);
-		digits = gtk_spin_button_get_value (GTK_SPIN_BUTTON (editor->priv->data_digits_spin));
-		if (editor->priv->data_format_fixed_flag) {
-			digits = gl_barcode_get_prefered_n(id);
-			gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->data_digits_spin), 
-						   digits);
-		}
+                digits = gl_barcode_get_prefered_n(id);
+                gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->data_digits_spin), digits);
                                                                                 
 		ex_string = gl_barcode_default_digits (id, digits);
 		gtk_label_set_text (GTK_LABEL(editor->priv->data_ex_label), ex_string);
@@ -208,14 +207,14 @@ style_changed_cb (glObjectEditor       *editor)
 						  !editor->priv->data_format_fixed_flag);
 		}
  
-                /* Emit our "changed" signal */
-                g_signal_emit (G_OBJECT (editor), gl_object_editor_signals[CHANGED], 0);
+                gl_object_editor_changed_cb (editor);
         }
                                                                                 
         g_free (style_string);
         g_free (ex_string);
 }
                                                                                 
+
 /*****************************************************************************/
 /* Set barcode style.                                                        */
 /*****************************************************************************/
@@ -235,8 +234,8 @@ gl_object_editor_set_bc_style (glObjectEditor      *editor,
 
         style_string = gl_barcode_id_to_name (id);
  
-	gl_util_combo_box_set_active_text (GTK_COMBO_BOX (editor->priv->bc_style_combo),
-					   style_string);
+	gl_combo_util_set_active_text (GTK_COMBO_BOX (editor->priv->bc_style_combo),
+                                       style_string);
  
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (editor->priv->bc_text_check),
                                       text_flag);
@@ -260,7 +259,7 @@ gl_object_editor_set_bc_style (glObjectEditor      *editor,
 	g_free (ex_string);
 
         gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->priv->data_digits_spin), 
-				   format_digits);
+                                   format_digits);
 
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->priv->data_literal_radio))) {
 		gtk_widget_set_sensitive (editor->priv->data_format_label, FALSE);
@@ -281,6 +280,7 @@ gl_object_editor_set_bc_style (glObjectEditor      *editor,
 	gl_debug (DEBUG_EDITOR, "END");
 }
 
+
 /*****************************************************************************/
 /* Query barcode style.                                                      */
 /*****************************************************************************/
@@ -314,6 +314,7 @@ gl_object_editor_get_bc_style (glObjectEditor      *editor,
 	gl_debug (DEBUG_EDITOR, "END");
 }
 
+
 /*****************************************************************************/
 /* Set bc color.                                                             */
 /*****************************************************************************/
@@ -322,8 +323,6 @@ gl_object_editor_set_bc_color (glObjectEditor      *editor,
 			       gboolean             merge_flag,
 			       glColorNode         *color_node)
 {
-	GdkColor *gdk_color;
-
 	gl_debug (DEBUG_EDITOR, "START");
 
         editor->priv->stop_signals = TRUE;
@@ -332,14 +331,12 @@ gl_object_editor_set_bc_color (glObjectEditor      *editor,
 
 	if ( color_node->color == GL_COLOR_NONE ) {
 
-		color_combo_set_color_to_default (COLOR_COMBO(editor->priv->bc_color_combo));
+		gl_color_combo_set_to_default (GL_COLOR_COMBO(editor->priv->bc_color_combo));
 
 	} else {
 
-		gdk_color = gl_color_to_gdk_color (color_node->color);
-		color_combo_set_color (COLOR_COMBO(editor->priv->bc_color_combo),
-					   gdk_color);
-		g_free (gdk_color);
+		gl_color_combo_set_color (GL_COLOR_COMBO(editor->priv->bc_color_combo),
+                                          color_node->color);
 	}
 	
 	if (!color_node->field_flag || !merge_flag) {
@@ -354,8 +351,8 @@ gl_object_editor_set_bc_color (glObjectEditor      *editor,
 		gtk_widget_set_sensitive (editor->priv->bc_color_combo, FALSE);
 		gtk_widget_set_sensitive (editor->priv->bc_key_combo, TRUE);
 		
-		gl_util_combo_box_set_active_text (GTK_COMBO_BOX (editor->priv->bc_key_combo),
-						   color_node->key);
+		gl_field_button_set_key (GL_FIELD_BUTTON (editor->priv->bc_key_combo),
+                                           color_node->key);
 	}	
 	
         editor->priv->stop_signals = FALSE;
@@ -363,13 +360,14 @@ gl_object_editor_set_bc_color (glObjectEditor      *editor,
 	gl_debug (DEBUG_EDITOR, "END");
 }
 
+
 /*****************************************************************************/
 /* Query bc color.                                                           */
 /*****************************************************************************/
 glColorNode*
 gl_object_editor_get_bc_color (glObjectEditor      *editor)
 {
-	GdkColor     *gdk_color;
+	guint         color;
 	glColorNode  *color_node;
 	gboolean      is_default;
 
@@ -379,19 +377,19 @@ gl_object_editor_get_bc_color (glObjectEditor      *editor)
 	
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->priv->bc_key_radio))) {
 		color_node->field_flag = TRUE;
-		color_node->color = gl_prefs->default_line_color;
+		color_node->color = gl_prefs_model_get_default_line_color (gl_prefs);
 		color_node->key = 
-			gtk_combo_box_get_active_text (GTK_COMBO_BOX (editor->priv->bc_key_combo));
+			gl_field_button_get_key (GL_FIELD_BUTTON (editor->priv->bc_key_combo));
 	} else {
 		color_node->field_flag = FALSE;
 		color_node->key = NULL;
-		gdk_color = color_combo_get_color (COLOR_COMBO(editor->priv->bc_color_combo),
-                                           &is_default);
+		color = gl_color_combo_get_color (GL_COLOR_COMBO(editor->priv->bc_color_combo),
+                                                  &is_default);
 
 		if (is_default) {
-			color_node->color = gl_prefs->default_line_color;
+			color_node->color = gl_prefs_model_get_default_line_color (gl_prefs);
 		} else {
-			color_node->color = gl_color_from_gdk_color (gdk_color);
+			color_node->color = color;
 		}
 	}
 	
@@ -400,6 +398,7 @@ gl_object_editor_get_bc_color (glObjectEditor      *editor)
 	return color_node;
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  barcode color radio callback.                                  */
 /*--------------------------------------------------------------------------*/
@@ -413,14 +412,51 @@ bc_radio_toggled_cb (glObjectEditor *editor)
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->priv->bc_color_radio))) {
                 gtk_widget_set_sensitive (editor->priv->bc_color_combo, TRUE);
                 gtk_widget_set_sensitive (editor->priv->bc_key_combo, FALSE);
-    } else {
+        } else {
                 gtk_widget_set_sensitive (editor->priv->bc_color_combo, FALSE);
                 gtk_widget_set_sensitive (editor->priv->bc_key_combo, TRUE);
 		
 	}
  
-    /* Emit our "changed" signal */
-    g_signal_emit (G_OBJECT (editor), gl_object_editor_signals[CHANGED], 0);
- 
-    gl_debug (DEBUG_EDITOR, "END");
+        gl_object_editor_changed_cb (editor);
 }
+
+
+/*--------------------------------------------------------------------------*/
+/* PRIVATE.  digits spin changed callback.                                  */
+/*--------------------------------------------------------------------------*/
+static void
+data_digits_spin_changed_cb (glObjectEditor *editor)
+{
+        gchar          *style_string;
+        const gchar    *id;
+        guint           digits;
+        gchar          *ex_string;
+
+        if (editor->priv->stop_signals) return;
+
+        style_string = gtk_combo_box_get_active_text (GTK_COMBO_BOX (editor->priv->bc_style_combo));
+        if ( *style_string != 0 ) {
+                id = gl_barcode_name_to_id (style_string);
+
+                digits = gtk_spin_button_get_value (GTK_SPIN_BUTTON (editor->priv->data_digits_spin));
+                ex_string = gl_barcode_default_digits (id, digits);
+                gtk_label_set_text (GTK_LABEL(editor->priv->data_ex_label), ex_string);
+        }
+
+        g_free (style_string);
+        g_free (ex_string);
+
+        gl_object_editor_changed_cb (editor);
+}
+
+
+
+/*
+ * Local Variables:       -- emacs
+ * mode: C                -- emacs
+ * c-basic-offset: 8      -- emacs
+ * tab-width: 8           -- emacs
+ * indent-tabs-mode: nil  -- emacs
+ * End:                   -- emacs
+ */
