@@ -1,25 +1,21 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
-
 /*
- *  (GLABELS) Label and Business Card Creation program for GNOME
+ *  xml-label.c
+ *  Copyright (C) 2001-2009  Jim Evins <evins@snaught.com>.
  *
- *  label.c:  GLabels xml label module
+ *  This file is part of gLabels.
  *
- *  Copyright (C) 2001-2002  Jim Evins <evins@snaught.com>.
- *
- *  This program is free software; you can redistribute it and/or modify
+ *  gLabels is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  gLabels is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ *  along with gLabels.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -27,11 +23,13 @@
 #include "xml-label.h"
 
 #include <glib/gi18n.h>
+#include <glib.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 #include <libxml/xinclude.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
 
+#include <libglabels.h>
 #include "label.h"
 #include "label-object.h"
 #include "label-text.h"
@@ -40,30 +38,32 @@
 #include "label-ellipse.h"
 #include "label-image.h"
 #include "label-barcode.h"
-#include "base64.h"
 #include "xml-label-04.h"
-#include <libglabels/db.h>
-#include <libglabels/xml-template.h>
-#include <libglabels/xml.h>
-#include "util.h"
+#include "str-util.h"
+#include "prefs.h"
 
 #include "debug.h"
 
+
 /*========================================================*/
 /* Private macros and constants.                          */
 /*========================================================*/
 #define COMPAT01_NAME_SPACE "http://snaught.com/glabels/0.1/"
 #define COMPAT04_NAME_SPACE "http://snaught.com/glabels/0.4/"
 #define COMPAT20_NAME_SPACE "http://snaught.com/glabels/2.0/"
+#define COMPAT22_NAME_SPACE "http://snaught.com/glabels/2.2/"
+
 
 /*========================================================*/
 /* Private types.                                         */
 /*========================================================*/
 
+
 /*========================================================*/
 /* Private globals.                                       */
 /*========================================================*/
 
+
 /*========================================================*/
 /* Private function prototypes.                           */
 /*========================================================*/
@@ -184,7 +184,7 @@ gl_xml_label_open (const gchar      *utf8_filename,
 	filename = g_filename_from_utf8 (utf8_filename, -1, NULL, NULL, NULL);
 	g_return_val_if_fail (filename, NULL);
 
-	doc = xmlParseFile (filename);
+        doc = xmlReadFile (filename, NULL, XML_PARSE_HUGE);
 	if (!doc) {
 		g_message (_("xmlParseFile error"));
 		*status = XML_LABEL_ERROR_OPEN_PARSE;
@@ -209,6 +209,7 @@ gl_xml_label_open (const gchar      *utf8_filename,
 	return label;
 }
 
+
 /****************************************************************************/
 /* Read label from xml buffer.                                              */
 /****************************************************************************/
@@ -221,7 +222,7 @@ gl_xml_label_open_buffer (const gchar      *buffer,
 
 	gl_debug (DEBUG_XML, "START");
 
-	doc = xmlParseDoc ((xmlChar *) buffer);
+        doc = xmlReadDoc ((xmlChar *) buffer, NULL, NULL, XML_PARSE_HUGE);
 	if (!doc) {
 		g_message (_("xmlParseFile error"));
 		*status = XML_LABEL_ERROR_OPEN_PARSE;
@@ -241,6 +242,7 @@ gl_xml_label_open_buffer (const gchar      *buffer,
 	return label;
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse xml doc structure and create label.                      */
 /*--------------------------------------------------------------------------*/
@@ -283,6 +285,7 @@ xml_doc_to_label (xmlDocPtr         doc,
 
         /* Test for current namespaces. */
         if ( !xmlSearchNsByHref (doc, root, (xmlChar *)COMPAT20_NAME_SPACE) &&
+             !xmlSearchNsByHref (doc, root, (xmlChar *)COMPAT22_NAME_SPACE) &&
              !xmlSearchNsByHref (doc, root, (xmlChar *)LGL_XML_NAME_SPACE) )
         {
                 g_message (_("Unknown glabels Namespace -- Using %s"),
@@ -300,6 +303,7 @@ xml_doc_to_label (xmlDocPtr         doc,
 	return label;
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse xml root node and create label.                          */
 /*--------------------------------------------------------------------------*/
@@ -343,7 +347,7 @@ xml_parse_label (xmlNodePtr        root,
 				return NULL;
 			}
 			lgl_db_register_template (template);
-			gl_label_set_template (label, template);
+			gl_label_set_template (label, template, FALSE);
 			lgl_template_free (template);
 		} else if (lgl_xml_is_node (child_node, "Objects")) {
 			xml_parse_objects (child_node, label);
@@ -367,6 +371,7 @@ xml_parse_label (xmlNodePtr        root,
 	return label;
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse Objects node.                                            */
 /*--------------------------------------------------------------------------*/
@@ -380,7 +385,7 @@ xml_parse_objects (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "START");
 
 	rotate_flag = lgl_xml_get_prop_boolean (node, "rotate", FALSE);
-	gl_label_set_rotate_flag (label, rotate_flag);
+	gl_label_set_rotate_flag (label, rotate_flag, FALSE);
 
 	for (child = node->xmlChildrenNode; child != NULL; child = child->next) {
 
@@ -407,6 +412,7 @@ xml_parse_objects (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML Objects->Object-text Node                            */
 /*--------------------------------------------------------------------------*/
@@ -424,27 +430,27 @@ xml_parse_object_text (xmlNodePtr  node,
 
 	gl_debug (DEBUG_XML, "START");
 
-	object = gl_label_text_new (label);
+	object = gl_label_text_new (label, FALSE);
 
 	/* position attrs */
 	x = lgl_xml_get_prop_length (node, "x", 0.0);
 	y = lgl_xml_get_prop_length (node, "y", 0.0);
-	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y);
+	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y, FALSE);
 
 	/* implied size attrs */
 	w = lgl_xml_get_prop_length (node, "w", 0);
 	h = lgl_xml_get_prop_length (node, "h", 0);
-	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h);
+	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h, FALSE);
 
 	/* justify attr */
 	string = lgl_xml_get_prop_string (node, "justify", NULL);
-	align = gl_util_string_to_align (string);
+	align = gl_str_util_string_to_align (string);
 	g_free (string);
-	gl_label_object_set_text_alignment (GL_LABEL_OBJECT(object), align);
+	gl_label_object_set_text_alignment (GL_LABEL_OBJECT(object), align, FALSE);
 
 	/* auto_shrink attr */
 	auto_shrink = lgl_xml_get_prop_boolean (node, "auto_shrink", FALSE);
-	gl_label_text_set_auto_shrink (GL_LABEL_TEXT(object), auto_shrink);
+	gl_label_text_set_auto_shrink (GL_LABEL_TEXT(object), auto_shrink, FALSE);
 
 	/* affine attrs */
 	xml_parse_affine_attrs (node, GL_LABEL_OBJECT(object));
@@ -468,6 +474,7 @@ xml_parse_object_text (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML Objects->Object-box Node                             */
 /*--------------------------------------------------------------------------*/
@@ -485,21 +492,21 @@ xml_parse_object_box (xmlNodePtr  node,
 
 	gl_debug (DEBUG_XML, "START");
 
-	object = gl_label_box_new (label);
+	object = gl_label_box_new (label, FALSE);
 
 	/* position attrs */
 	x = lgl_xml_get_prop_length (node, "x", 0.0);
 	y = lgl_xml_get_prop_length (node, "y", 0.0);
-	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y);
+	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y, FALSE);
 
 	/* size attrs */
 	w = lgl_xml_get_prop_length (node, "w", 0);
 	h = lgl_xml_get_prop_length (node, "h", 0);
-	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h);
+	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h, FALSE);
 
 	/* line attrs */
 	line_width = lgl_xml_get_prop_length (node, "line_width", 1.0);
-	gl_label_object_set_line_width (GL_LABEL_OBJECT(object), line_width);
+	gl_label_object_set_line_width (GL_LABEL_OBJECT(object), line_width, FALSE);
 	
 	line_color_node = gl_color_node_new_default ();
 	string = lgl_xml_get_prop_string (node, "line_color_field", NULL);
@@ -509,7 +516,7 @@ xml_parse_object_box (xmlNodePtr  node,
 	} else {
 		line_color_node->color = lgl_xml_get_prop_uint (node, "line_color", 0);
 	}
-	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color_node);
+	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color_node, FALSE);
 	gl_color_node_free (&line_color_node);
 
 
@@ -522,7 +529,7 @@ xml_parse_object_box (xmlNodePtr  node,
 	} else {
 		fill_color_node->color = lgl_xml_get_prop_uint (node, "fill_color", 0);
 	}
-	gl_label_object_set_fill_color (GL_LABEL_OBJECT(object), fill_color_node);
+	gl_label_object_set_fill_color (GL_LABEL_OBJECT(object), fill_color_node, FALSE);
 	gl_color_node_free (&fill_color_node);
 	
 	/* affine attrs */
@@ -534,6 +541,7 @@ xml_parse_object_box (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML Objects->Object-ellipse Node                         */
 /*--------------------------------------------------------------------------*/
@@ -551,21 +559,21 @@ xml_parse_object_ellipse (xmlNodePtr  node,
 
 	gl_debug (DEBUG_XML, "START");
 
-	object = gl_label_ellipse_new (label);
+	object = gl_label_ellipse_new (label, FALSE);
 
 	/* position attrs */
 	x = lgl_xml_get_prop_length (node, "x", 0.0);
 	y = lgl_xml_get_prop_length (node, "y", 0.0);
-	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y);
+	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y, FALSE);
 
 	/* size attrs */
 	w = lgl_xml_get_prop_length (node, "w", 0);
 	h = lgl_xml_get_prop_length (node, "h", 0);
-	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h);
+	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h, FALSE);
 
 	/* line attrs */
 	line_width = lgl_xml_get_prop_length (node, "line_width", 1.0);
-	gl_label_object_set_line_width (GL_LABEL_OBJECT(object), line_width);
+	gl_label_object_set_line_width (GL_LABEL_OBJECT(object), line_width, FALSE);
 
 	line_color_node = gl_color_node_new_default ();
 	string = lgl_xml_get_prop_string (node, "line_color_field", NULL);
@@ -575,7 +583,7 @@ xml_parse_object_ellipse (xmlNodePtr  node,
 	} else {
 		line_color_node->color = lgl_xml_get_prop_uint (node, "line_color", 0);		
 	}
-	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color_node);
+	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color_node, FALSE);
 	gl_color_node_free (&line_color_node);
 
 
@@ -588,7 +596,7 @@ xml_parse_object_ellipse (xmlNodePtr  node,
 	} else {
 		fill_color_node->color = lgl_xml_get_prop_uint (node, "fill_color", 0);
 	}
-	gl_label_object_set_fill_color (GL_LABEL_OBJECT(object), fill_color_node);
+	gl_label_object_set_fill_color (GL_LABEL_OBJECT(object), fill_color_node, FALSE);
 	gl_color_node_free (&fill_color_node);
 
 	/* affine attrs */
@@ -600,6 +608,7 @@ xml_parse_object_ellipse (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML Objects->Object-line Node                            */
 /*--------------------------------------------------------------------------*/
@@ -616,21 +625,21 @@ xml_parse_object_line (xmlNodePtr  node,
 
 	gl_debug (DEBUG_XML, "START");
 
-	object = gl_label_line_new (label);
+	object = gl_label_line_new (label, FALSE);
 
 	/* position attrs */
 	x = lgl_xml_get_prop_length (node, "x", 0.0);
 	y = lgl_xml_get_prop_length (node, "y", 0.0);
-	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y);
+	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y, FALSE);
 
 	/* length attrs */
 	dx = lgl_xml_get_prop_length (node, "dx", 0);
 	dy = lgl_xml_get_prop_length (node, "dy", 0);
-	gl_label_object_set_size (GL_LABEL_OBJECT(object), dx, dy);
+	gl_label_object_set_size (GL_LABEL_OBJECT(object), dx, dy, FALSE);
 
 	/* line attrs */
 	line_width = lgl_xml_get_prop_length (node, "line_width", 1.0);
-	gl_label_object_set_line_width (GL_LABEL_OBJECT(object), line_width);
+	gl_label_object_set_line_width (GL_LABEL_OBJECT(object), line_width, FALSE);
 	
 	line_color_node = gl_color_node_new_default ();
 	string = lgl_xml_get_prop_string (node, "line_color_field", NULL);
@@ -640,7 +649,7 @@ xml_parse_object_line (xmlNodePtr  node,
 	} else {
 		line_color_node->color = lgl_xml_get_prop_uint (node, "line_color", 0);		
 	}
-	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color_node);
+	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), line_color_node, FALSE);
 	gl_color_node_free (&line_color_node);
 
 	/* affine attrs */
@@ -652,6 +661,7 @@ xml_parse_object_line (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML Objects->Object-image Node                           */
 /*--------------------------------------------------------------------------*/
@@ -667,12 +677,12 @@ xml_parse_object_image (xmlNodePtr  node,
 
 	gl_debug (DEBUG_XML, "START");
 
-	object = gl_label_image_new (label);
+	object = gl_label_image_new (label, FALSE);
 
 	/* position attrs */
 	x = lgl_xml_get_prop_length (node, "x", 0.0);
 	y = lgl_xml_get_prop_length (node, "y", 0.0);
-	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y);
+	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y, FALSE);
 
 	/* src or field attr */
 	string = lgl_xml_get_prop_string (node, "src", NULL);
@@ -680,7 +690,7 @@ xml_parse_object_image (xmlNodePtr  node,
 		filename = g_new0 (glTextNode, 1);
 		filename->field_flag = FALSE;
 		filename->data = g_strdup ((gchar *)string);
-		gl_label_image_set_filename (GL_LABEL_IMAGE(object), filename);
+		gl_label_image_set_filename (GL_LABEL_IMAGE(object), filename, FALSE);
 		gl_text_node_free (&filename);
 		xmlFree (string);
 	} else {
@@ -689,7 +699,7 @@ xml_parse_object_image (xmlNodePtr  node,
 			filename = g_new0 (glTextNode, 1);
 			filename->field_flag = TRUE;
 			filename->data = g_strdup ((gchar *)string);
-			gl_label_image_set_filename (GL_LABEL_IMAGE(object), filename);
+			gl_label_image_set_filename (GL_LABEL_IMAGE(object), filename, FALSE);
 			gl_text_node_free (&filename);
 			xmlFree (string);
 		} else {
@@ -700,7 +710,7 @@ xml_parse_object_image (xmlNodePtr  node,
 	/* size attrs */
 	w = lgl_xml_get_prop_length (node, "w", 0);
 	h = lgl_xml_get_prop_length (node, "h", 0);
-	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h);
+	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h, FALSE);
 
 	/* affine attrs */
 	xml_parse_affine_attrs (node, GL_LABEL_OBJECT(object));
@@ -711,6 +721,7 @@ xml_parse_object_image (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML Objects->Object-barcode Node                         */
 /*--------------------------------------------------------------------------*/
@@ -731,17 +742,17 @@ xml_parse_object_barcode (xmlNodePtr  node,
 
 	gl_debug (DEBUG_XML, "START");
 
-	object = gl_label_barcode_new (label);
+	object = gl_label_barcode_new (label, FALSE);
 
 	/* position attrs */
 	x = lgl_xml_get_prop_length (node, "x", 0.0);
 	y = lgl_xml_get_prop_length (node, "y", 0.0);
-	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y);
+	gl_label_object_set_position (GL_LABEL_OBJECT(object), x, y, FALSE);
 
 	/* size attrs */
 	w = lgl_xml_get_prop_length (node, "w", 0);
 	h = lgl_xml_get_prop_length (node, "h", 0);
-	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h);
+	gl_label_object_set_size (GL_LABEL_OBJECT(object), w, h, FALSE);
 
 	/* prop attrs */
 	id = lgl_xml_get_prop_string (node, "style", NULL);
@@ -749,7 +760,7 @@ xml_parse_object_barcode (xmlNodePtr  node,
 	checksum_flag = lgl_xml_get_prop_boolean (node, "checksum", TRUE);
 	format_digits = lgl_xml_get_prop_uint (node, "format", 10);
 	gl_label_barcode_set_props (GL_LABEL_BARCODE(object),
-				    (gchar *)id, text_flag, checksum_flag, format_digits);
+				    (gchar *)id, text_flag, checksum_flag, format_digits, FALSE);
 	g_free (id);
 	
 	color_node = gl_color_node_new_default ();
@@ -760,7 +771,7 @@ xml_parse_object_barcode (xmlNodePtr  node,
 	} else {
 		color_node->color = lgl_xml_get_prop_uint (node, "color", 0);		
 	}
-	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), color_node);
+	gl_label_object_set_line_color (GL_LABEL_OBJECT(object), color_node, FALSE);
 	gl_color_node_free (&color_node);
 
 	/* data or field attr */
@@ -769,7 +780,7 @@ xml_parse_object_barcode (xmlNodePtr  node,
 		text_node = g_new0 (glTextNode, 1);
 		text_node->field_flag = FALSE;
 		text_node->data = string;
-		gl_label_barcode_set_data (GL_LABEL_BARCODE(object), text_node);
+		gl_label_barcode_set_data (GL_LABEL_BARCODE(object), text_node, FALSE);
 		gl_text_node_free (&text_node);
 	} else {
 		string = lgl_xml_get_prop_string (node, "field", NULL);
@@ -777,7 +788,7 @@ xml_parse_object_barcode (xmlNodePtr  node,
 			text_node = g_new0 (glTextNode, 1);
 			text_node->field_flag = TRUE;
 			text_node->data = string;
-			gl_label_barcode_set_data (GL_LABEL_BARCODE(object), text_node);
+			gl_label_barcode_set_data (GL_LABEL_BARCODE(object), text_node, FALSE);
 			gl_text_node_free (&text_node);
 		} else {
 			g_message ("Missing Object-barcode data or field attr");
@@ -793,6 +804,7 @@ xml_parse_object_barcode (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML merge fields tag.                                    */
 /*--------------------------------------------------------------------------*/
@@ -815,7 +827,7 @@ xml_parse_merge_fields (xmlNodePtr  node,
                 gl_merge_set_src (merge, string);
                 g_free (string);
 
-                gl_label_set_merge (label, merge);
+                gl_label_set_merge (label, merge, FALSE);
 
                 g_object_unref (G_OBJECT(merge));
         }
@@ -823,6 +835,7 @@ xml_parse_merge_fields (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML data tag.                                            */
 /*--------------------------------------------------------------------------*/
@@ -849,6 +862,7 @@ xml_parse_data (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse XML pixbuf data tag.                                     */
 /*--------------------------------------------------------------------------*/
@@ -858,7 +872,7 @@ xml_parse_pixdata (xmlNodePtr  node,
 {
 	gchar      *name, *base64;
 	guchar     *stream;
-	guint       stream_length;
+	gsize       stream_length;
 	gboolean    ret;
 	GdkPixdata *pixdata;
 	GdkPixbuf  *pixbuf;
@@ -869,7 +883,7 @@ xml_parse_pixdata (xmlNodePtr  node,
 	name = lgl_xml_get_prop_string (node, "name", NULL);
 	base64 = lgl_xml_get_node_content (node);
 
-	stream = gl_base64_decode ((gchar *)base64, &stream_length);
+	stream = g_base64_decode ((gchar *)base64, &stream_length);
 	pixdata = g_new0 (GdkPixdata, 1);
 	ret = gdk_pixdata_deserialize (pixdata, stream_length, stream, NULL);
 
@@ -889,6 +903,7 @@ xml_parse_pixdata (xmlNodePtr  node,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse top-level Span tag.                                      */
 /*--------------------------------------------------------------------------*/
@@ -911,22 +926,22 @@ xml_parse_toplevel_span  (xmlNodePtr        node,
 
 	/* Font family attr */
 	font_family = lgl_xml_get_prop_string (node, "font_family", "Sans");
-	gl_label_object_set_font_family (object, font_family);
+	gl_label_object_set_font_family (object, font_family, FALSE);
 	g_free (font_family);
 
 	/* Font size attr */
 	font_size = lgl_xml_get_prop_double (node, "font_size", 0.0);
-	gl_label_object_set_font_size (object, font_size);
+	gl_label_object_set_font_size (object, font_size, FALSE);
 
 	/* Font weight attr */
 	string = lgl_xml_get_prop_string (node, "font_weight", NULL);
-	font_weight = gl_util_string_to_weight (string);
+	font_weight = gl_str_util_string_to_weight (string);
 	g_free (string);
-	gl_label_object_set_font_weight (object, font_weight);
+	gl_label_object_set_font_weight (object, font_weight, FALSE);
 
 	/* Font italic flag attr */
 	font_italic_flag = lgl_xml_get_prop_boolean (node, "font_italic", FALSE);
-	gl_label_object_set_font_italic_flag (object, font_italic_flag);
+	gl_label_object_set_font_italic_flag (object, font_italic_flag, FALSE);
 
 	/* Text color attr */
 	color_node = gl_color_node_new_default ();
@@ -937,13 +952,12 @@ xml_parse_toplevel_span  (xmlNodePtr        node,
 	} else {
 		color_node->color = lgl_xml_get_prop_uint (node, "color", 0);		
 	}
-	gl_label_object_set_text_color (object, color_node);
+	gl_label_object_set_text_color (object, color_node, FALSE);
 	gl_color_node_free (&color_node);
 	
-
 	/* Text line spacing attr  */
 	text_line_spacing = lgl_xml_get_prop_double (node, "line_spacing", 1.0);
-	gl_label_object_set_text_line_spacing (object, text_line_spacing); 
+	gl_label_object_set_text_line_spacing (object, text_line_spacing, FALSE); 
 
 	/* Now descend children, and build lines of text nodes */
 	lines = NULL;
@@ -992,12 +1006,13 @@ xml_parse_toplevel_span  (xmlNodePtr        node,
 		lines = g_list_append (lines, text_nodes);
 		text_nodes = NULL;
 	}
-	gl_label_text_set_lines (GL_LABEL_TEXT(object), lines);
+	gl_label_text_set_lines (GL_LABEL_TEXT(object), lines, FALSE);
 	gl_text_node_lines_free (&lines);
 
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse affine attributes.                                       */
 /*--------------------------------------------------------------------------*/
@@ -1020,6 +1035,7 @@ xml_parse_affine_attrs (xmlNodePtr        node,
 	gl_label_object_set_matrix (object, &matrix);
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Parse shadow attributes.                                       */
 /*--------------------------------------------------------------------------*/
@@ -1035,13 +1051,13 @@ xml_parse_shadow_attrs (xmlNodePtr        node,
 	gchar           *string;
 
 	shadow_state = lgl_xml_get_prop_boolean (node, "shadow", FALSE);
-	gl_label_object_set_shadow_state (object, shadow_state);
+	gl_label_object_set_shadow_state (object, shadow_state, FALSE);
 
 	if (shadow_state)
 	{
 		shadow_x = lgl_xml_get_prop_length (node, "shadow_x", 0.0);
 		shadow_y = lgl_xml_get_prop_length (node, "shadow_y", 0.0);
-		gl_label_object_set_shadow_offset (object, shadow_x, shadow_y);
+		gl_label_object_set_shadow_offset (object, shadow_x, shadow_y, FALSE);
 		
 		shadow_color_node = gl_color_node_new_default ();
 		string = lgl_xml_get_prop_string (node, "shadow_color_field", NULL);
@@ -1051,14 +1067,15 @@ xml_parse_shadow_attrs (xmlNodePtr        node,
 		} else {
 			shadow_color_node->color = lgl_xml_get_prop_uint (node, "shadow_color", 0);		
 		}
-		gl_label_object_set_shadow_color (object, shadow_color_node);
+		gl_label_object_set_shadow_color (object, shadow_color_node, FALSE);
 		gl_color_node_free (&shadow_color_node);
 
 		shadow_opacity = lgl_xml_get_prop_double (node, "shadow_opacity", 1.0);
-		gl_label_object_set_shadow_opacity (object, shadow_opacity);
+		gl_label_object_set_shadow_opacity (object, shadow_opacity, FALSE);
 	}
 }
 
+
 /****************************************************************************/
 /* Save label to xml label file.                                            */
 /****************************************************************************/
@@ -1099,6 +1116,7 @@ gl_xml_label_save (glLabel          *label,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /****************************************************************************/
 /* Save label to xml buffer.                                                */
 /****************************************************************************/
@@ -1124,6 +1142,7 @@ gl_xml_label_save_buffer (glLabel          *label,
 	return (gchar *)buffer;
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Convert label to xml doc structure.                            */
 /*--------------------------------------------------------------------------*/
@@ -1131,21 +1150,27 @@ static xmlDocPtr
 xml_label_to_doc (glLabel          *label,
 		  glXMLLabelStatus *status)
 {
-	xmlDocPtr   doc;
-	xmlNsPtr    ns;
-	glMerge    *merge;
+        lglUnits           units;
+	xmlDocPtr          doc;
+	xmlNsPtr           ns;
+        const lglTemplate *template;
+	glMerge           *merge;
 
 	gl_debug (DEBUG_XML, "START");
 
 	LIBXML_TEST_VERSION;
 
+        units = gl_prefs_model_get_units (gl_prefs);
+        lgl_xml_set_default_units (units);
+
 	doc = xmlNewDoc ((xmlChar *)"1.0");
 	doc->xmlRootNode = xmlNewDocNode (doc, NULL, (xmlChar *)"Glabels-document", NULL);
 
 	ns = xmlNewNs (doc->xmlRootNode, (xmlChar *)LGL_XML_NAME_SPACE, NULL);
 	xmlSetNs (doc->xmlRootNode, ns);
 
-	lgl_xml_template_create_template_node (label->template, doc->xmlRootNode, ns);
+        template = gl_label_get_template (label);
+	lgl_xml_template_create_template_node (template, doc->xmlRootNode, ns);
 
 	xml_create_objects (doc->xmlRootNode, ns, label);
 
@@ -1164,6 +1189,7 @@ xml_label_to_doc (glLabel          *label,
 	return doc;
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Objects Node                                           */
 /*--------------------------------------------------------------------------*/
@@ -1173,16 +1199,21 @@ xml_create_objects (xmlNodePtr  root,
 		    glLabel    *label)
 {
 	xmlNodePtr     node;
+        gboolean       rotate_flag;
+        const GList   *object_list;
 	GList         *p;
 	glLabelObject *object;
 
 	gl_debug (DEBUG_XML, "START");
 
+        rotate_flag = gl_label_get_rotate_flag (label);
+        object_list = gl_label_get_object_list (label);
+
 	node = xmlNewChild (root, ns, (xmlChar *)"Objects", NULL);
 	lgl_xml_set_prop_string (node, "id", "0");
-	lgl_xml_set_prop_boolean (node, "rotate", label->rotate_flag);
+	lgl_xml_set_prop_boolean (node, "rotate", rotate_flag);
 
-	for (p = label->objects; p != NULL; p = p->next) {
+	for (p = (GList *)object_list; p != NULL; p = p->next) {
 
 		object = GL_LABEL_OBJECT(p->data);
 
@@ -1207,6 +1238,7 @@ xml_create_objects (xmlNodePtr  root,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Objects->Object-text Node                              */
 /*--------------------------------------------------------------------------*/
@@ -1237,7 +1269,7 @@ xml_create_object_text (xmlNodePtr     root,
 
 	/* justify attr */
 	align = gl_label_object_get_text_alignment (object);
-	lgl_xml_set_prop_string (node, "justify", gl_util_align_to_string (align));
+	lgl_xml_set_prop_string (node, "justify", gl_str_util_align_to_string (align));
 
 	/* auto_shrink attr */
 	auto_shrink = gl_label_text_get_auto_shrink (GL_LABEL_TEXT (object));
@@ -1255,6 +1287,7 @@ xml_create_object_text (xmlNodePtr     root,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Objects->Object-box Node                               */
 /*--------------------------------------------------------------------------*/
@@ -1320,6 +1353,7 @@ xml_create_object_box (xmlNodePtr     root,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Objects->Object-ellipse Node                           */
 /*--------------------------------------------------------------------------*/
@@ -1386,6 +1420,7 @@ xml_create_object_ellipse (xmlNodePtr     root,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Objects->Object-line Node                              */
 /*--------------------------------------------------------------------------*/
@@ -1439,6 +1474,7 @@ xml_create_object_line (xmlNodePtr     root,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Objects->Object-image Node                             */
 /*--------------------------------------------------------------------------*/
@@ -1484,6 +1520,7 @@ xml_create_object_image (xmlNodePtr     root,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Objects->Object-barcode Node                           */
 /*--------------------------------------------------------------------------*/
@@ -1556,6 +1593,7 @@ xml_create_object_barcode (xmlNodePtr     root,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Label Merge Fields Node                                */
 /*--------------------------------------------------------------------------*/
@@ -1587,6 +1625,7 @@ xml_create_merge_fields (xmlNodePtr  root,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Label Data Node                                        */
 /*--------------------------------------------------------------------------*/
@@ -1616,6 +1655,7 @@ xml_create_data (xmlNodePtr  root,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Add XML Label Data Pixbuf Node                                 */
 /*--------------------------------------------------------------------------*/
@@ -1643,7 +1683,7 @@ xml_create_pixdata (xmlNodePtr  root,
 		pixdata = g_new0 (GdkPixdata, 1);
 		gdk_pixdata_from_pixbuf (pixdata, pixbuf, FALSE);
 		stream = gdk_pixdata_serialize (pixdata, &stream_length);
-		base64 = gl_base64_encode (stream, stream_length);
+		base64 = g_base64_encode (stream, stream_length);
 
 		node = xmlNewChild (root, ns, (xmlChar *)"Pixdata", (xmlChar *)base64);
 		lgl_xml_set_prop_string (node, "name", name);
@@ -1660,6 +1700,7 @@ xml_create_pixdata (xmlNodePtr  root,
 	gl_debug (DEBUG_XML, "END");
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Create top-level Span node.                                    */
 /*--------------------------------------------------------------------------*/
@@ -1703,7 +1744,7 @@ xml_create_toplevel_span (xmlNodePtr        root,
 	align = gl_label_object_get_text_alignment (GL_LABEL_OBJECT(object_text));
 	lgl_xml_set_prop_string (node, "font_family", font_family);
 	lgl_xml_set_prop_double (node, "font_size", font_size);
-	lgl_xml_set_prop_string (node, "font_weight", gl_util_weight_to_string (font_weight));
+	lgl_xml_set_prop_string (node, "font_weight", gl_str_util_weight_to_string (font_weight));
 	lgl_xml_set_prop_boolean (node, "font_italic", font_italic_flag);
 	
 	lgl_xml_set_prop_double (node, "line_spacing", text_line_spacing);
@@ -1736,6 +1777,7 @@ xml_create_toplevel_span (xmlNodePtr        root,
 
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Create affine attributes.                                      */
 /*--------------------------------------------------------------------------*/
@@ -1755,6 +1797,7 @@ xml_create_affine_attrs (xmlNodePtr        node,
 	lgl_xml_set_prop_double (node, "a5", matrix.y0);
 }
 
+
 /*--------------------------------------------------------------------------*/
 /* PRIVATE.  Create shadow attributes.                                      */
 /*--------------------------------------------------------------------------*/
@@ -1795,3 +1838,12 @@ xml_create_shadow_attrs (xmlNodePtr        node,
 }
 
 
+
+/*
+ * Local Variables:       -- emacs
+ * mode: C                -- emacs
+ * c-basic-offset: 8      -- emacs
+ * tab-width: 8           -- emacs
+ * indent-tabs-mode: nil  -- emacs
+ * End:                   -- emacs
+ */
